@@ -19,11 +19,12 @@ use super::event::Event;
 use super::program::Program;
 use super::Result;
 
+#[allow(unused_imports)]
+use cl3::ext;
 use cl3::types::{
     cl_context, cl_device_id, cl_event, cl_kernel, cl_kernel_exec_info, cl_program, cl_uint,
     cl_ulong,
 };
-
 use libc::{c_void, size_t};
 use std::ffi::CString;
 use std::mem;
@@ -31,6 +32,7 @@ use std::ptr;
 
 /// An OpenCL kernel object.  
 /// Implements the Drop trait to call release_kernel when the object is dropped.
+#[derive(Debug)]
 pub struct Kernel {
     kernel: cl_kernel,
 }
@@ -251,6 +253,40 @@ impl Kernel {
         )?
         .to_ulong())
     }
+
+    #[cfg(feature = "cl_khr_subgroups")]
+    pub fn get_kernel_sub_group_info_khr(
+        &self,
+        device: cl_device_id,
+        device: cl_device_id,
+        param_name: ext::KernelSubGroupInfoKhr,
+        input_values: &[size_t],
+    ) -> Result<size_t> {
+        Ok(get_kernel_sub_group_info_khr(
+            self.kernel,
+            device,
+            param_name,
+            input_values.len(),
+            input_values.as_ptr(),
+        )?)
+    }
+
+    #[cfg(feature = "cl_khr_suggested_local_work_size")]
+    pub fn get_kernel_suggested_local_work_size_khr(
+        &self,
+        command_queue: cl_command_queue,
+        work_dim: cl_uint,
+        global_work_offset: *const size_t,
+        global_work_size: *const size_t,
+    ) -> Result<size_t> {
+        Ok(get_kernel_suggested_local_work_size_khr(
+            command_queue,
+            self.kernel,
+            work_dim,
+            global_work_offset,
+            global_work_size,
+        )?)
+    }
 }
 
 /// Create OpenCL Kernel objects for all the kernel functions in a program.
@@ -270,6 +306,7 @@ pub fn create_program_kernels(program: &Program) -> Result<Vec<Kernel>> {
 /// A struct that implements the [builder pattern](https://doc.rust-lang.org/1.0.0/style/ownership/builders.html)
 /// to simplify setting up [Kernel] arguments and the [NDRange](https://www.khronos.org/registry/OpenCL/specs/3.0-unified/html/OpenCL_API.html#_mapping_work_items_onto_an_ndrange)
 /// when enqueueing a [Kernel] on a [CommandQueue].
+#[derive(Debug)]
 pub struct ExecuteKernel<'a> {
     pub kernel: &'a Kernel,
     pub num_args: cl_uint,
