@@ -14,21 +14,39 @@
 
 use core::marker::PhantomData;
 
+#[allow(unused_imports)]
+pub use cl3::ffi::cl_ext::cl_mem_properties_intel;
 pub use cl3::memory::*;
 
 use super::context::Context;
 
 use super::Result;
 #[allow(unused_imports)]
+use cl3::d3d10;
+#[allow(unused_imports)]
+use cl3::d3d11;
+#[allow(unused_imports)]
+use cl3::dx9_media_sharing;
+#[allow(unused_imports)]
 use cl3::egl;
+#[allow(unused_imports)]
+use cl3::ext;
+#[allow(unused_imports)]
+use cl3::ffi::cl_d3d10::{ID3D10Buffer_ptr, ID3D10Texture2D_ptr, ID3D10Texture3D_ptr};
+#[allow(unused_imports)]
+use cl3::ffi::cl_d3d11::{ID3D11Buffer_ptr, ID3D11Texture2D_ptr, ID3D11Texture3D_ptr};
+#[allow(unused_imports)]
+use cl3::ffi::cl_dx9_media_sharing::{
+    cl_dx9_media_adapter_type_khr, IDirect3DSurface9_ptr, HANDLE,
+};
 use cl3::gl;
 use cl3::memory;
 use cl3::sampler;
 #[allow(unused_imports)]
 use cl3::types::{
-    cl_addressing_mode, cl_bool, cl_buffer_create_type, cl_context, cl_filter_mode, cl_image_desc,
-    cl_image_format, cl_int, cl_mem, cl_mem_flags, cl_mem_object_type, cl_mem_properties,
-    cl_sampler, cl_sampler_properties, cl_uint, cl_ulong,
+    cl_addressing_mode, cl_bool, cl_buffer_create_type, cl_buffer_region, cl_context,
+    cl_filter_mode, cl_image_desc, cl_image_format, cl_int, cl_mem, cl_mem_flags,
+    cl_mem_object_type, cl_mem_properties, cl_sampler, cl_sampler_properties, cl_uint, cl_ulong,
 };
 use libc::{c_void, intptr_t, size_t};
 use std::mem;
@@ -98,6 +116,7 @@ pub trait ClMem {
 
 /// An OpenCL buffer.  
 /// Implements the Drop trait to call release_mem_object when the object is dropped.
+#[derive(Debug)]
 pub struct Buffer<T> {
     buffer: cl_mem,
     #[doc(hidden)]
@@ -198,25 +217,51 @@ impl<T> Buffer<T> {
         Ok(Buffer::new(buffer))
     }
 
+    #[cfg(feature = "cl_intel_create_buffer_with_properties")]
+    pub fn create_with_properties_intel(
+        context: &Context,
+        properties: *const cl_mem_properties_intel,
+        flags: cl_mem_flags,
+        count: size_t,
+        host_ptr: *mut c_void,
+    ) -> Result<Buffer<T>> {
+        let buffer = ext::create_buffer_with_properties_intel(
+            context.get(),
+            properties,
+            flags,
+            count * mem::size_of::<T>(),
+            host_ptr,
+        )?;
+        Ok(Buffer::new(buffer))
+    }
+
     /// Create an new OpenCL buffer object from an existing buffer object.  
+    /// See: [SubBuffer Attributes](https://www.khronos.org/registry/OpenCL/specs/3.0-unified/html/OpenCL_API.html#subbuffer-create-info-table).  
     ///
     /// * `flags` - a bit-field used to specify allocation and usage information
     /// about the sub-buffer memory object being created, see:
     /// [Memory Flags](https://www.khronos.org/registry/OpenCL/specs/3.0-unified/html/OpenCL_API.html#memory-flags-table).
-    /// * `buffer_create_type`,`buffer_create_info` - describe the type of
-    /// buffer object to be created, see:
-    /// [SubBuffer Attributes](https://www.khronos.org/registry/OpenCL/specs/3.0-unified/html/OpenCL_API.html#subbuffer-create-info-table).
+    /// * `origin` - the offset in number of objects of type <T>.
+    /// * `count` - the size of the sub-buffer in number of objects of type <T>.
     ///
     /// returns a Result containing the new OpenCL buffer object
     /// or the error code from the OpenCL C API function.
     pub fn create_sub_buffer(
         &self,
         flags: cl_mem_flags,
-        buffer_create_type: cl_buffer_create_type,
-        buffer_create_info: *const c_void,
+        origin: usize,
+        count: usize,
     ) -> Result<Buffer<T>> {
-        let buffer =
-            memory::create_sub_buffer(self.buffer, flags, buffer_create_type, buffer_create_info)?;
+        let buffer_create_info = cl_buffer_region {
+            origin: origin * std::mem::size_of::<T>(),
+            size: count * std::mem::size_of::<T>(),
+        };
+        let buffer = memory::create_sub_buffer(
+            self.buffer,
+            flags,
+            CL_BUFFER_CREATE_TYPE_REGION,
+            &buffer_create_info as *const _ as *const c_void,
+        )?;
         Ok(Buffer::new(buffer))
     }
 }
@@ -225,6 +270,7 @@ impl<T> Buffer<T> {
 /// Has methods to return information from calls to clGetImageInfo with the
 /// appropriate parameters.  
 /// Implements the Drop trait to call release_mem_object when the object is dropped.
+#[derive(Debug)]
 pub struct Image {
     image: cl_mem,
 }
@@ -431,6 +477,118 @@ impl Image {
         Ok(Image::new(image))
     }
 
+    #[cfg(feature = "cl_khr_dx9_media_sharing")]
+    #[inline]
+    pub fn create_from_dx9_media_surface_khr(
+        context: &Context,
+        flags: cl_mem_flags,
+        adapter_type: cl_dx9_media_adapter_type_khr,
+        surface_info: *mut c_void,
+        plane: cl_uint,
+    ) -> Result<Image> {
+        let image = dx9_media_sharing::create_from_dx9_media_surface_khr(
+            context.get(),
+            flags,
+            adapter_type,
+            surface_info,
+            plane,
+        )?;
+        Ok(Image::new(image))
+    }
+
+    #[cfg(feature = "cl_intel_dx9_media_sharing")]
+    #[inline]
+    pub fn create_from_dx9_media_surface_intel(
+        context: &Context,
+        flags: cl_mem_flags,
+        resource: IDirect3DSurface9_ptr,
+        shared_handle: HANDLE,
+        plane: cl_uint,
+    ) -> Result<Image> {
+        let image = dx9_media_sharing::create_from_dx9_media_surface_intel(
+            context.get(),
+            flags,
+            resource,
+            shared_handle,
+            plane,
+        )?;
+        Ok(Image::new(image))
+    }
+
+    #[cfg(feature = "cl_khr_d3d10_sharing")]
+    #[inline]
+    pub fn create_from_d3d10_buffer_khr(
+        context: &Context,
+        flags: cl_mem_flags,
+        resource: ID3D10Buffer_ptr,
+    ) -> Result<Image> {
+        let image = d3d10::create_from_d3d10_buffer_khr(context.get(), flags, resource)?;
+        Ok(Image::new(image))
+    }
+
+    #[cfg(feature = "cl_khr_d3d10_sharing")]
+    #[inline]
+    pub fn create_from_d3d10_texture2d_khr(
+        context: &Context,
+        flags: cl_mem_flags,
+        resource: ID3D10Texture2D_ptr,
+        subresource: cl_uint,
+    ) -> Result<Image> {
+        let image =
+            d3d10::create_from_d3d10_texture2d_khr(context.get(), flags, resource, subresource)?;
+        Ok(Image::new(image))
+    }
+
+    #[cfg(feature = "cl_khr_d3d10_sharing")]
+    #[inline]
+    pub fn create_from_d3d10_texture3d_khr(
+        context: &Context,
+        flags: cl_mem_flags,
+        resource: ID3D10Texture3D_ptr,
+        subresource: cl_uint,
+    ) -> Result<Image> {
+        let image =
+            d3d10::create_from_d3d10_texture3d_khr(context.get(), flags, resource, subresource)?;
+        Ok(Image::new(image))
+    }
+
+    #[cfg(feature = "cl_khr_d3d11_sharing")]
+    #[inline]
+    pub fn create_from_d3d11_buffer_khr(
+        context: &Context,
+        flags: cl_mem_flags,
+        resource: ID3D11Buffer_ptr,
+    ) -> Result<Image> {
+        let image = d3d11::create_from_d3d11_buffer_khr(context.get(), flags, resource)?;
+        Ok(Image::new(image))
+    }
+
+    #[cfg(feature = "cl_khr_d3d11_sharing")]
+    #[inline]
+    pub fn create_from_d3d11_texture2d_khr(
+        context: &Context,
+        flags: cl_mem_flags,
+        resource: ID3D11Texture2D_ptr,
+        subresource: cl_uint,
+    ) -> Result<Image> {
+        let image =
+            d3d11::create_from_d3d11_texture2d_khr(context.get(), flags, resource, subresource)?;
+        Ok(Image::new(image))
+    }
+
+    #[cfg(feature = "cl_khr_d3d11_sharing")]
+    #[inline]
+    pub fn create_from_d3d11_texture3d_khr(
+        context: &Context,
+        flags: cl_mem_flags,
+        resource: ID3D11Texture3D_ptr,
+        subresource: cl_uint,
+    ) -> Result<Image> {
+        let image =
+            d3d11::create_from_d3d11_texture3d_khr(context.get(), flags, resource, subresource)?;
+        Ok(Image::new(image))
+    }
+
     pub fn format(&self) -> Result<Vec<cl_image_format>> {
         Ok(memory::get_image_info(self.image, ImageInfo::CL_IMAGE_FORMAT)?.to_vec_image_format())
     }
@@ -494,6 +652,7 @@ impl Image {
 /// Has methods to return information from calls to clGetSamplerInfo with the
 /// appropriate parameters.  
 /// Implements the Drop trait to call release_sampler when the object is dropped.
+#[derive(Debug)]
 pub struct Sampler {
     sampler: cl_sampler,
 }
@@ -541,6 +700,7 @@ impl Sampler {
 /// Has methods to return information from calls to clGetPipeInfo with the
 /// appropriate parameters.  
 /// Implements the Drop trait to call release_mem_object when the object is dropped.
+#[derive(Debug)]
 pub struct Pipe {
     pipe: cl_mem,
 }
