@@ -19,6 +19,8 @@ use super::context::Context;
 use super::Result;
 #[allow(unused_imports)]
 use cl3::error_codes::CL_BUILD_PROGRAM_FAILURE;
+#[allow(unused_imports)]
+use cl3::ext;
 use cl3::types::{cl_context, cl_device_id, cl_int, cl_program, cl_uchar, cl_uint};
 use libc::{intptr_t, size_t};
 use std::ffi::{CStr, CString};
@@ -76,6 +78,7 @@ pub const ENABLE_LINK_OPTIONS: &str = "-enable-link-options ";
 /// An OpenCL program object.  
 /// Stores the names of the OpenCL kernels in the program.
 /// Implements the Drop trait to call release_program when the object is dropped.
+#[derive(Debug)]
 pub struct Program {
     program: cl_program,
     kernel_names: String,
@@ -188,7 +191,18 @@ impl Program {
     /// or the error code from the OpenCL C API function.
     #[cfg(feature = "CL_VERSION_2_1")]
     pub fn create_from_il(context: &Context, il: &[u8]) -> Result<Program> {
-        Ok(Program::new(create_program_with_il(context.get(), &il)?))
+        Ok(Program::new(
+            create_program_with_il(context.get(), &il)?,
+            "",
+        ))
+    }
+
+    #[cfg(feature = "cl_khr_il_program")]
+    pub fn create_from_il_khr(context: &Context, il: &[u8]) -> Result<Program> {
+        Ok(Program::new(
+            ext::create_program_with_il_khr(context.get(), &il)?,
+            "",
+        ))
     }
 
     /// Build (compile & link) a Program.
